const assert = require("assert");
const webdriver = require("selenium-webdriver");
const conf = require("../selenium.conf");

module.exports = async function() {
	this.capability.name += `: ${this.scriptName}`;
	this.capability.pageLoadStrategy = this.pageLoadStrategy;
	const driver = await new webdriver.Builder()
		.usingServer(conf.address)
		.withCapabilities(this.capability)
		.build();

	let passed = 0;
	const total = 3;

	try {
		await driver.get(process.env.CI_ADDRESS);
		if (this.resourceUrl) {
			await driver.executeAsyncScript(`
			const materialScript = document.createElement("script");
			materialScript.setAttribute("src", "${this.resourceUrl}");
			window.document.head.appendChild(materialScript);
			setTimeout(arguments[arguments.length - 1], 10000);
			`);
		}

		const sidebarColor = await driver.findElement(webdriver.By.css("#mcmap .sidebar")).getCssValue("background-color");
		assert.strictEqual(/\(33, 33, 33\b/.test(sidebarColor), true, "Sidebar background not applied");
		passed++;

		const chipContainer = await driver.findElement(webdriver.By.css("#mcmap .leaflet-chip-container"));
		assert.strictEqual(chipContainer instanceof webdriver.WebElement, true, "Chip component not present");
		passed++;

		const minimapContainer = await driver.findElement(webdriver.By.css("#mcmap .leaflet-minimap-container"));
		assert.strictEqual(minimapContainer instanceof webdriver.WebElement, true, "Minimap component not present");
		passed++;

		await driver.executeScript(
			`browserstack_executor: {"action": "setSessionStatus", "arguments": {"status": "passed", "reason": "All tests have passed (${passed}/${total} OK)" } }`
		);
	} catch (e) {
		await driver.executeScript(
			`browserstack_executor: {"action": "setSessionStatus", "arguments": {"status": "failed", "reason": "Some tests were not successful (${passed}/${total} OK)" } }`
		);
		throw new Error(`Some tests were not successful (${passed}/${total} OK)`);
	}

	this.capability.name = this.capability.browserName;
	await driver.quit();
};
